# Generated by Django 2.2.8 on 2019-12-14 17:12

from django.db import migrations


class Migration(migrations.Migration):

    atomic = False
    
    dependencies = [
        ('freight', '0013_auto_20191214_1522'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='EveOrganization',
            new_name='EveEntity',
        ),
    ]
